from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.policy.policy_runners.common_policy_runner import CommonPolicyRunner


@logger_time_stamp
def run_common_policies():
    """
    This method run the common policies
    :return:
    """
    common_policy_runner = CommonPolicyRunner()
    common_policy_runner.run()



import boto3
import typeguard

from cloud_governance.common.clouds.aws.athena.abstract_athena_operations import AbstractAthenaOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp


class BotoClientAthenaOperations(AbstractAthenaOperations):
    """
    This class performs the aws athena operations
    """

    def __init__(self):
        super().__init__()
        self.__athena_client = boto3.client('athena', region_name='us-east-1')

    @typeguard.typechecked
    @logger_time_stamp
    def execute_query(self, query_string: str):
        """
        This method executes the query and returns the s3_path, QueryExecutionId
        Limit: Continuously check the s3 bucket that file is created or not using the QueryExecutionId
        :param query_string:
        :return:
        """
        try:
            s3_path = self.__s3_results_path
            logger.debug(f"Query Output path: s3_path/{self.CURRENT_DATE}")
            result = self.__athena_client.start_query_execution(
                QueryString=query_string,
                ResultConfiguration={
                    "OutputLocation": s3_path,
                }
            )
            if result:
                bucket, key = self.__s3_results_path.replace("s3://", "").split('/')
                return {
                    'QueryExecutionId': result.get('QueryExecutionId'),
                    's3_key': s3_path,
                    's3_csv_path': f'{s3_path}/{result.get("QueryExecutionId")}.csv',
                    'bucket': bucket,
                    'key': f'{key}/{self.CURRENT_DATE}',
                    'file_name': f'{result.get("QueryExecutionId")}.csv'
                }
        except Exception as err:
            logger.error(err)
        return {}



from ast import literal_eval

import pandas as pd

from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix
from cloud_governance.main.environment_variables import environment_variables


class MonthlyReport:
    REPORT_DAYS = 30

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self._es_index = 'cloud-governance-mail-messages'
        self._es_host = self.__environment_variables_dict.get('es_host', '')
        self._es_port = self.__environment_variables_dict.get('es_port', '')
        self._to_mail = literal_eval(self.__environment_variables_dict.get('to_mail', '[]'))
        self._to_cc = literal_eval(self.__environment_variables_dict.get('cc_mail', '[]'))
        if self._es_host:
            self._elastic_operations = ElasticSearchOperations(es_host=self._es_host, es_port=self._es_port)
        self._postfix_mail = Postfix()
        self._mail_message = MailMessage()

    def policy_description(self, policy_name: str):
        """
        This method returns the policy description
        @param policy_name:
        @return:
        """
        policy_descriptions = {
            'ec2_stop': 'Delete the stopped instances that are stopped for more than 30 days ',
            'instance_idle': 'stops the idle instances in the last 7 days.  ( CPU < 5%, Network < 5k )',
            'unattached_volume': 'Delete  unattached EBS volumes.',
            'ip_unattached': 'Delete all the elastic_ips that are unused',
            'unused_nat_gateway': ' Delete all unused nat gateways',
            'zombie_snapshots': 'Delete all the snapshots which the AMI does not use',
            's3_inactive': 'Delete the empty buckets which don’t have any content.',
            'empty_roles': 'Delete the empty role which does\'t have any policies',
            'zombie_cluster_resource': 'Delete up the cluster resources which are not deleted while cleaning the cluster'
        }
        return policy_descriptions.get(policy_name)

    def get_monthly_report_data(self):
        """
        This method fetch the last 30 days data from ElasticSearch
        @return:
        """
        if self._es_host:
            mail_messages = self._elastic_operations.get_index_hits(days=self.REPORT_DAYS, index=self._es_index)
            mail_messages_df = pd.DataFrame(mail_messages).drop(['To', 'Cc', 'Message'], axis=1).dropna()
            mail_messages_df = mail_messages_df[~mail_messages_df.MessageType.str.contains('notify_admin')].reset_index(drop=True)
            mail_messages_df = mail_messages_df[~mail_messages_df.MessageType.str.contains('monthly_report')].reset_index(drop=True)
            mail_messages_df_group = mail_messages_df.groupby(['Policy', 'Account']).agg({'MessageType': list}).reset_index()
            return mail_messages_df_group.to_dict("records")
        else:
            logger.info('es_host is missing')
        return []

    def send_monthly_report(self):
        """
        This method send monthly report to the user
        @return:
        """
        monthly_data = self.get_monthly_report_data()
        prepare_data = {"Perf-Dept".upper(): [], "Perf-Scale".upper(): []}
        for data in monthly_data:
            account_name = data['Account']
            if account_name:
                prepare_data[account_name].append({
                    "Policy": data['Policy'],
                    "Alerts": len(data['MessageType'])
                })
        content = self.prepare_html_table_message(prepare_data)
        if content:
            subject, body = self._mail_message.monthly_html_mail_message(data=content)
            self._postfix_mail.send_email_postfix(subject=subject, content=body, to=self._to_mail, cc=self._to_cc, mime_type='html', message_type='monthly_report')
            return 'Successfully Send the Monthly report'
        return 'No Data to send to the Monthly report'

    def row_span(self, cols: int):
        """
        This method returns the table data with colspan
        @param cols:
        @return:
        """
        return f'<td rowspan="{cols}" align="center" style="border: 1px solid black;font-weight: bold;color:blue">'

    def prepare_html_table_message(self, data: dict):
        """
        This method gives the html table from dictionary
        @param data:
        @return:
        """
        start_row, end_row = '<tr>', '</tr>'
        start_col, end_col = f'<td align="left" style="border:1px solid black;font-weight:bold;">', '</b></td>'
        start_head, end_head = f'<th align="left" style="border:1px solid black;color:white;background:gray;padding:2px;font-size:12px;">', '</th>'
        table_keys = ['Policy', 'Description', 'Alerts']
        html_table = [f"""
                    <table style="border-collapse:collapse;width: auto">
                        <caption style="caption-side:bottom;text-align: left;"><span style="color:red"><b>Note:</b></span> All policies follow 4 days alert 7 days delete</caption>
                        {start_row}
                            {start_head}Account{end_head}
                            {start_head}Policy{end_head}
                            {start_head}Description{end_head}
                            {start_head}Alerts{end_head}
                        {end_row}
                      """.strip()]
        for data_key, data_values in data.items():
            row_span = len(data_values)
            html_row = start_row
            if data_key == 'PERF-SCALE':
                data_key = 'Openshift-PerfScale'
            html_row += f'{self.row_span(row_span)}{data_key}{end_col}'
            for data_items in data_values:
                data_items['Description'] = self.policy_description(policy_name=data_items.get('Policy'))
                print(data_items)
                for table_key in table_keys:
                    if table_key == "Alerts":
                        html_row += f'{start_col[:-2]}color:green;"> {str(data_items.get(table_key))} {end_col}'
                    else:
                        html_row += f'{start_col} {str(data_items.get(table_key))} {end_col}'
                html_row += end_row
                html_table.append(html_row)
                html_row = start_row
        html_table.append('</tbody></table>')
        return "\n".join(html_table)

    def run(self):
        """
        This method sends the monthly report
        @return:
        """
        return self.send_monthly_report()



import datetime
import json

from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.common.google_drive.gcp_operations import GCPOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.utils.api_requests import APIRequests
from cloud_governance.common.utils.configs import LOOK_BACK_DAYS, PUBLIC_ACCOUNTS_COST_REPORTS, MONTHS, \
    DEFAULT_ROUND_DIGITS, DATE_FORMAT
from cloud_governance.common.utils.utils import Utils
from cloud_governance.main.environment_variables import environment_variables


class CloudabilityCostReports:
    """
    This class performs cloudability cost operations
    """

    APPITO_LOGIN_API = "https://frontdoor.apptio.com/service/apikeylogin"

    def __init__(self):
        self.__api_requests = APIRequests()
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__view_id = self.__environment_variables_dict.get('CLOUDABILITY_VIEW_ID')
        self.__dimensions = self.__environment_variables_dict.get('CLOUDABILITY_DIMENSIONS')
        self.__metrics = self.__environment_variables_dict.get('CLOUDABILITY_METRICS')
        self.__cloudability_api = self.__environment_variables_dict.get('CLOUDABILITY_API')
        self.__reports_path = self.__environment_variables_dict.get('CLOUDABILITY_API_REPORTS_PATH')
        self.__appito_envid = self.__environment_variables_dict.get('APPITO_ENVID')
        self.__key_secret = self.__environment_variables_dict.get('APPITO_KEY_SECRET')
        self.__key_access = self.__environment_variables_dict.get('APPITO_KEY_ACCESS')
        self.__gcp_operations = GCPOperations()
        self.elastic_upload = ElasticUpload()

    def __get_appito_token(self):
        """
        This method returns the appito token
        :return:
        :rtype:
        """
        data = {
            "keyAccess": self.__key_access,
            "keySecret": self.__key_secret
        }
        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }
        response = self.__api_requests.post(url=self.APPITO_LOGIN_API, data=json.dumps(data), headers=headers)
        if response.ok:
            return response.headers['apptio-opentoken']
        return None

    def __get_start_date(self):
        return (self.__get_end_date() - datetime.timedelta(days=LOOK_BACK_DAYS)).replace(day=1)

    def __get_end_date(self):
        return datetime.datetime.utcnow().date()

    def __get_cost_reports(self, start_date: str = None, end_date: str = None, custom_filter: str = ''):
        """
        This method returns the cost reports from the cloudability
        :return:
        :rtype:
        """
        appito_token = self.__get_appito_token()
        if not appito_token:
            raise Exception("Appito Token missing error")
        if not start_date:
            start_date = self.__get_start_date()
        if not end_date:
            end_date = self.__get_end_date()
        api = (f'{self.__cloudability_api}/{self.__reports_path}?'
               f'dimensions={self.__dimensions}&metrics={self.__metrics}'
               f'&start_date={start_date}&end_date={end_date}'
               f'&id={self.__view_id}&{custom_filter}')
        headers = {
            "apptio-environmentid": self.__appito_envid,
            "apptio-opentoken": appito_token
        }

        response = self.__api_requests.get(url=api, headers=headers)
        if isinstance(response, dict):
            return response.get('results')
        return {}

    def __get_analysed_reports(self):
        """
        This method returns the cost reports by adding actual usage, allocated budget
        :return:
        :rtype:
        """
        accounts_reports_df = self.__gcp_operations.get_accounts_sheet(sheet_name=PUBLIC_ACCOUNTS_COST_REPORTS)
        cost_centers = list(set(accounts_reports_df['CostCenter'].tolist()))
        custom_filter = '&'.join([f'filters=category4=={cost_center}' for cost_center in cost_centers])
        cloudability_reports = self.__get_cost_reports(custom_filter=custom_filter)
        cost_reports = {}
        for account in cloudability_reports:
            account['date'] = (datetime.datetime.strptime(account.get('date', ''), DATE_FORMAT)
                               .replace(day=1).date().__str__())
            account_id = account.get('vendor_account_identifier').replace('-', '')
            account_row = accounts_reports_df[accounts_reports_df['AccountId'] == account_id].reset_index().to_dict(
                orient='records')
            if account_row:
                timestamp = datetime.datetime.strptime(account.get('date'), '%Y-%m-%d')
                account_data = account_row[0]
                cost_center = account_data.get('CostCenter', 0)
                account_budget = round(float(account_data.get('Budget', '0').replace(',', '')))
                year = str(account_data.get('Year'))
                account_owner = account_data.get('Owner')
                cloud_name = account.get('vendor').upper()
                if Utils.equal_ignore_case(cloud_name, 'amazon'):
                    cloud_name = 'AWS'
                month = datetime.datetime.strftime(timestamp, '%Y %b')
                index_id = f"""cloudability-{account.get('date')}-{account.get('vendor_account_name').lower()}"""

                if year in account.get('date'):
                    if index_id not in cost_reports:
                        cost_reports.setdefault(index_id, {}).update({
                            'Account': account.get('vendor_account_name'),
                            'Actual': round(float(account.get('unblended_cost')), DEFAULT_ROUND_DIGITS),
                            'AccountId': account_id,
                            'Owner': account_owner,
                            'start_date': account.get('date'),
                            'CloudName': cloud_name,
                            'Forecast': 0,
                            'index_id': index_id,
                            'timestamp': timestamp,
                            'Month': month,
                            'Budget': round(account_budget / MONTHS, DEFAULT_ROUND_DIGITS),
                            'AllocatedBudget': account_budget,
                            'CostCenter': cost_center,
                            'filter_date': f'{account.get("date")}-{month.split()[-1]}'
                        })
                    else:
                        cost_reports[index_id]['Actual'] += round(float(account.get('unblended_cost')), DEFAULT_ROUND_DIGITS)
        return list(cost_reports.values())

    def __next_twelve_months(self):
        """
        This method returns the next 12 months, year
        :return:
        """
        year = datetime.datetime.utcnow().year
        next_month = datetime.datetime.utcnow().month + 1
        month_year = []
        for idx in range(MONTHS):
            month = str((idx + next_month) % MONTHS)
            c_year = year
            if len(month) == 1:
                month = f'0{month}'
            if month == '00':
                month = 12
                year = year+1
            month_year.append((str(month), c_year))
        return month_year

    def __forecast_for_next_months(self, cost_data: list):
        """
        This method returns the forecast of next twelve months data
        :param cost_data:
        :return:
        """
        forecast_cost_data = []
        month_years = self.__next_twelve_months()
        month = (datetime.datetime.utcnow().month - 1) % 12
        if month == 0:
            month = 12
        if len(str(month)) == 1:
            month = f'0{month}'
        year = datetime.datetime.utcnow().year
        cache_start_date = f'{year}-{str(month)}-01'
        for data in cost_data:
            if cache_start_date == data.get('start_date') and data.get('CostCenter') > 0:
                for m_y in month_years:
                    m, y = m_y[0], m_y[1]
                    start_date = f'{y}-{m}-01'
                    timestamp = datetime.datetime.strptime(start_date, "%Y-%m-%d")
                    index_id = f'cloudability-{start_date}-{data.get("Account").lower()}'
                    month = datetime.datetime.strftime(timestamp, "%Y %b")
                    forecast_cost_data.append({
                        **data,
                        'Actual': 0,
                        'start_date': start_date,
                        'timestamp': timestamp,
                        'index_id': index_id,
                        'filter_date': f'{start_date}-{month.split()[-1]}',
                        'Month': month}
                    )
        return forecast_cost_data

    def __get_cost_and_upload(self):
        """
        This method collect the cost and uploads to the ElasticSearch
        :return:
        """
        collected_data = self.__get_analysed_reports()
        forecast_data = self.__forecast_for_next_months(cost_data=collected_data)
        upload_data = collected_data + forecast_data
        self.elastic_upload.es_upload_data(items=upload_data, set_index='index_id')
        return upload_data

    def run(self):
        """
        This is the starting of the operations
        :return:
        :rtype:
        """
        logger.info(f'Cloudability Cost Reports=> ReportsPath: {self.__reports_path}, Metrics: {self.__metrics}, API: {self.__cloudability_api}')
        self.__get_cost_and_upload()




from cloud_governance.policy.helpers.azure.azure_policy_operations import AzurePolicyOperations


class IpUnattached(AzurePolicyOperations):

        RESOURCE_ACTION = "Delete"

        def __init__(self):
            super().__init__()
            self.__network_interfaces = self.network_operations.get_public_ipv4_network_interfaces()

        def __check_ipv4_not_associated(self, ipv4_address_dict: dict):
            """
            This method returns bool
            :param ipv4_address_dict:
            :type ipv4_address_dict:
            :return:
            :rtype:
            """
            found = False
            ip_address_id = ipv4_address_dict.get('id')
            if not ipv4_address_dict.get('ip_configuration'):
                found = True
            else:
                network_interface_id = ipv4_address_dict.get('ip_configuration', {}).get('id')
                if ip_address_id in self.__network_interfaces:
                    for network_interface in self.__network_interfaces.get(ip_address_id):
                        if network_interface.get('id') in network_interface_id:
                            if not network_interface.get('virtual_machine'):
                                found = True
            return found

        def run_policy_operations(self, volume=None):
            """
            This method returns the list of unattached IPV4's
            :return:
            :rtype:
            """
            unattached_ips = []
            active_cluster_ids = self._get_active_cluster_ids()
            public_ipv4_address = self.network_operations.get_public_ipv4_addresses()
            for ip_address in public_ipv4_address:
                tags = ip_address.get('tags')
                cleanup_result = False
                is_disassociated_ipv4 = False
                cluster_tag = self._get_cluster_tag(tags=tags)
                if cluster_tag not in active_cluster_ids:
                    is_disassociated_ipv4 = self.__check_ipv4_not_associated(ip_address)
                    if is_disassociated_ipv4:
                        cleanup_days = self.get_clean_up_days_count(tags=tags)
                        cleanup_result = self.verify_and_delete_resource(resource_id=ip_address.get('id'), tags=tags,
                                                                         clean_up_days=cleanup_days)
                        resource_data = self._get_es_schema(resource_id=ip_address.get('name'),
                                                            user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                                                            skip_policy=self.get_skip_policy_value(tags=tags),
                                                            cleanup_days=cleanup_days, dry_run=self._dry_run,
                                                            name=ip_address.get('name'), region=ip_address.get('location'),
                                                            cleanup_result=str(cleanup_result),
                                                            resource_action=self.RESOURCE_ACTION,
                                                            cloud_name=self._cloud_name,
                                                            resource_type="PublicIPv4 Static",
                                                            resource_state="disassociated" if not cleanup_result else "Deleted"
                                                            )
                        unattached_ips.append(resource_data)
                else:
                    cleanup_days = 0
                if not cleanup_result:
                    if self.get_tag_name_from_tags(tags, tag_name='DaysCount') or is_disassociated_ipv4:
                        self.update_resource_day_count_tag(resource_id=ip_address.get("id"),
                                                           cleanup_days=cleanup_days, tags=tags)
            return unattached_ips




from cloud_governance.policy.policy_operations.aws.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy


class EbsInUse(NonClusterZombiePolicy):

    def __init__(self):
        super(EbsInUse, self).__init__()

    def run(self):
        """
        This method list all in-use ebs volumes
        @return:
        """
        in_use_volumes = []
        volumes = self._ec2_operations.get_volumes()
        for volume in volumes:
            if volume.get('State') == 'in-use':
                in_use_volumes.append(volume)
        return self._organise_instance_data(in_use_volumes)



import datetime

from dateutil.relativedelta import relativedelta

from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp

from cloud_governance.common.google_drive.upload_to_gsheet import UploadToGsheet

from cloud_governance.common.clouds.ibm.account.ibm_account import IBMAccount
from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.main.environment_variables import environment_variables


class CostBillingReports:
    """
    This class is responsible for generation cost billing report for Budget, Actual, Forecast
    """

    MONTHS = 12

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__ibm_account = IBMAccount()
        self.__elastic_upload = ElasticUpload()
        self.update_to_gsheet = UploadToGsheet()
        self.cost_center, self.__account_budget, self.__years, self.__owner = self.update_to_gsheet.get_cost_center_budget_details(account_id=self.__ibm_account.short_account_id)

    def prepare_es_data(self, month: str, year: str, usage_cost: float = 0, next_invoice: float = 0):
        """This method prepares the data to upload to the es"""
        start_date = f'{year}-{month}-01'
        timestamp = datetime.datetime.strptime(start_date, '%Y-%m-%d')
        month = datetime.datetime.strftime(timestamp, '%Y %b')
        es_data = {
            'AccountId': self.__ibm_account.short_account_id,
            'Account': self.__ibm_account.account.lower(),
            'start_date': timestamp,
            'index_id': f'{start_date}-{self.__ibm_account.short_account_id}',
            'timestamp': timestamp,
            'Budget': round(self.__account_budget/self.MONTHS, 3),
            'AllocatedBudget': self.__account_budget,
            'Month': month,
            'CostCenter': self.cost_center,
            'CloudName': 'IBM Cloud',
            'Owner': self.__owner,
            'Forecast': round(next_invoice, 3),
            'Actual': round(usage_cost, 3),
            'filter_date': f'{start_date}-{month.split()[-1]}',
        }
        return es_data

    def get_cost_usage_details(self):
        """This method fetch cost usage details"""
        date = datetime.datetime.now()
        month, year = date.month, date.year
        upload_es_data = {}
        usage_data = self.__ibm_account.get_daily_usage(month=month, year=year)
        if usage_data:
            usage_cost = round(usage_data.get('resources').get('billable_cost'), 3)
            next_invoice = self.__ibm_account.get_next_recurring_invoice()
            es_data = self.prepare_es_data(usage_cost=usage_cost, next_invoice=next_invoice, month=date.strftime("%m"), year=str(year))
            upload_es_data[es_data['index_id']] = es_data
        past_month = date - relativedelta(month=month-1)
        last_month, last_month_year = past_month.strftime("%m"), past_month.year
        past_usage_cost = self.__ibm_account.get_daily_usage(month=int(last_month), year=last_month_year)
        if past_usage_cost:
            es_data = self.prepare_es_data(usage_cost=round(past_usage_cost.get('resources').get('billable_cost'), 3), month=str(last_month), year=str(last_month_year))
            upload_es_data[es_data['index_id']] = es_data
        for next_month in range(self.MONTHS):
            next_month = (next_month + month) % self.MONTHS
            if next_month != month:
                c_year = year
                if len(str(next_month)) != 2:
                    next_month = f'0{next_month}'
                if next_month == '00':
                    year += 1
                    next_month = str(12)
                es_data = self.prepare_es_data(month=str(next_month), year=str(c_year))
                upload_es_data[es_data['index_id']] = es_data
        if upload_es_data:
            self.__elastic_upload.es_upload_data(items=list(upload_es_data.values()), set_index='index_id')
        return list(upload_es_data.values())

    def run(self):
        """This method run the billing methods"""
        self.get_cost_usage_details()



import typeguard

from cloud_governance.common.logger.init_logger import logger


class DeleteS3Resources:
    """
    This class deleted the zombie resources of s3
    """

    def __init__(self, s3_client, s3_resource):
        """
        Initialize the AWS clients
        :param s3_client:
        :param s3_resource:
        """
        self.s3_client = s3_client
        self.s3_resource = s3_resource

    @typeguard.typechecked
    def delete_zombie_s3_resource(self, resource_type: str, resource_id: str):
        """
        This method checks the  which resource to delete
        :param resource_type:
        :param resource_id:
        :return:
        """
        if resource_type == 's3_bucket':
            self.__delete_s3_bucket(resource_id=resource_id)

    @typeguard.typechecked
    def __delete_s3_bucket(self, resource_id: str):
        """
        This method delete the bucket from s3
        :param resource_id:
        :return:
        """
        try:
            # delete bucket objects
            bucket = self.s3_resource.Bucket(resource_id)
            bucket.objects.all().delete()
            # delete bucket
            self.s3_client.delete_bucket(Bucket=resource_id)
            logger.info(f'delete_bucket: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_bucket: {resource_id}, {err}')




import importlib
import inspect

from typing import Callable

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.policy_runners.aws.upload_s3 import UploadS3
from cloud_governance.policy.policy_runners.common.abstract_policy_runner import AbstractPolicyRunner


class PolicyRunner(AbstractPolicyRunner):

    def __init__(self):
        super().__init__()

    def execute_policy(self, policy_class_name: str, run_policy: Callable, upload: bool):
        """
        This method executes the policy
        :param policy_class_name:
        :type policy_class_name:
        :param run_policy:
        :type run_policy:
        :param upload:
        :type upload:
        :return:
        :rtype:
        """
        policy_result = []
        ec2_operations = EC2Operations()
        upload_to_s3 = UploadS3()
        active_regions = [self._region]
        if self._run_active_regions:
            active_regions = ec2_operations.get_active_regions()
            logger.info("Running the policy in All AWS active regions")
        for active_region in active_regions:
            logger.info(f"Running the {self._policy} in Region: {active_region}")
            self._environment_variables_dict['AWS_DEFAULT_REGION'] = active_region
            response = run_policy().run()
            if isinstance(response, str):
                logger.info(f'key: {policy_class_name}, Response: {response}')
            else:
                policy_result.extend(response)
                logger.info(f'key: {policy_class_name}, count: {len(response)}, {response}')
                if upload:
                    self._upload_elastic_search.upload(data=response)
                    upload_to_s3.upload(data=response)
        return policy_result



from azure.mgmt.compute import ComputeManagementClient
from azure.mgmt.compute.v2023_03_01.models import VirtualMachine

from cloud_governance.common.clouds.azure.common.common_operations import CommonOperations
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp


class ComputeOperations(CommonOperations):

    def __init__(self):
        super().__init__()
        self.__compute_client = ComputeManagementClient(self._default_creds, subscription_id=self._subscription_id)

    @logger_time_stamp
    def get_instances_by_resource_group(self, resource_group_name: str) -> [VirtualMachine]:
        """
        This method returns all the compute resources by resource group
        :return:
        """
        instances_paged_object = self.__compute_client.virtual_machines.list(resource_group_name=resource_group_name)
        instances_list: [VirtualMachine] = self._item_paged_iterator(item_paged_object=instances_paged_object)
        return instances_list

    def get_all_instances(self) -> [VirtualMachine]:
        """
        This method returns all the virtual machines
        :return:
        :rtype:
        """
        instances_paged_object = self.__compute_client.virtual_machines.list_all()
        instances_list: [VirtualMachine] = self._item_paged_iterator(item_paged_object=instances_paged_object)
        return instances_list

    def get_instance_statuses(self, resource_id: str, vm_name: str) -> dict:
        """
        This method returns the virtual machine instance status
        :param vm_name:
        :type vm_name:
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        resource_group_name = self.get_resource_group_name_from_resource_id(resource_id=resource_id)
        virtual_machine = self.__compute_client.virtual_machines.instance_view(resource_group_name=resource_group_name,
                                                                               vm_name=vm_name)
        return virtual_machine.as_dict()

    def stop_vm(self, resource_id: str):
        """
        This method stops the vm
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        id_key_pairs = self.get_id_dict_data(resource_id)
        resource_group_name = id_key_pairs.get('resourcegroups')
        vm_name = id_key_pairs.get('virtualmachines')
        status = self.__compute_client.virtual_machines.begin_deallocate(resource_group_name=resource_group_name,
                                                                         vm_name=vm_name)
        return status.done()

    # volumes -> disks
    def get_all_disks(self):
        """
        This method returns all the disks
        :return:
        :rtype:
        """
        paged_volumes = self.__compute_client.disks.list()
        return self._item_paged_iterator(item_paged_object=paged_volumes, as_dict=True)

    def delete_disk(self, resource_id: str):
        """
        This method deletes the disk
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        id_key_pairs = self.get_id_dict_data(resource_id)
        resource_group_name = id_key_pairs.get('resourcegroups')
        disk_name = id_key_pairs.get('disks')
        status = self.__compute_client.disks.begin_delete(resource_group_name=resource_group_name, disk_name=disk_name)
        return status.done()
